(define-module (my-slim)
  #:use-module (gnu artwork)
  #:use-module (gnu services)
  #:use-module (gnu services shepherd)
  #:use-module (gnu services xorg)
  #:use-module (gnu system pam)
  #:use-module (gnu system keyboard)
  #:use-module (gnu services base)
  #:use-module (gnu services dbus)
  #:use-module ((gnu packages base) #:select (canonical-package))
  #:use-module (gnu packages guile)
  #:use-module (gnu packages xorg)
  #:use-module (gnu packages fonts)
  #:use-module (gnu packages gl)
  #:use-module (gnu packages glib)
  #:use-module (gnu packages display-managers)
  #:use-module (gnu packages freedesktop)
  #:use-module (gnu packages gnustep)
  #:use-module (gnu packages gnome)
  #:use-module (gnu packages admin)
  #:use-module (gnu packages bash)
  #:use-module (gnu system shadow)
  #:use-module (guix gexp)
  #:use-module (guix store)
  #:use-module (guix packages)
  #:use-module (guix derivations)
  #:use-module (guix records)
  #:use-module (guix deprecation)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-9)
  #:use-module (srfi srfi-26)
  #:use-module (ice-9 match)
  #:export (my-slim-configuration
            my-slim-configuration?
            my-slim-configuration-slim
            my-slim-configuration-allow-empty-passwords?
            my-slim-configuration-auto-login?
            my-slim-configuration-default-user
            my-slim-configuration-theme
            my-slim-configuration-theme-name
            my-slim-configuration-xauth
            my-slim-configuration-shepherd
            my-slim-configuration-auto-login-session
            my-slim-configuration-xorg
            my-slim-configuration-display
            my-slim-configuration-vt
            my-slim-configuration-sessreg

            my-slim-service-type))

(define-record-type* <my-slim-configuration>
  my-slim-configuration make-my-slim-configuration
  my-slim-configuration?
  (slim my-slim-configuration-slim
        (default slim))
  (allow-empty-passwords? my-slim-configuration-allow-empty-passwords?
                          (default #t))
  (auto-login? my-slim-configuration-auto-login?
               (default #f))
  (default-user my-slim-configuration-default-user
                (default ""))
  (theme my-slim-configuration-theme
         (default %default-slim-theme))
  (theme-name my-slim-configuration-theme-name
              (default %default-slim-theme-name))
  (xauth my-slim-configuration-xauth
         (default xauth))
  (shepherd my-slim-configuration-shepherd
            (default shepherd))
  (auto-login-session my-slim-configuration-auto-login-session
                      (default #f))
  (xorg-configuration my-slim-configuration-xorg
                      (default (xorg-configuration)))
  (display my-slim-configuration-display
           (default ":0"))
  (vt my-slim-configuration-vt
      (default "vt7"))
  (sessreg my-slim-configuration-sessreg
           (default sessreg)))

(define (my-slim-pam-service config)
  "Return a PAM service for @command{slim}."
  (list (unix-pam-service
         "slim"
         #:allow-empty-passwords?
         (my-slim-configuration-allow-empty-passwords? config))))

(define (my-slim-shepherd-service config)
  (let ((vt (my-slim-configuration-vt config)))
    (define slim.cfg
      (let ((xinitrc (xinitrc #:fallback-session
                              (my-slim-configuration-auto-login-session config)))
            (slim    (my-slim-configuration-slim config))
            (xauth   (my-slim-configuration-xauth config))
            (startx  (xorg-start-command (my-slim-configuration-xorg config)))
            (args (string-append (my-slim-configuration-display config)
                                 " "
                                 vt))
            (shepherd   (my-slim-configuration-shepherd config))
            (theme-name (my-slim-configuration-theme-name config))
            (sessreg (my-slim-configuration-sessreg config)))
        (mixed-text-file "slim.cfg"  "
default_path /run/current-system/profile/bin
default_xserver " startx "
xserver_arguments " args "
xauth_path " xauth "/bin/xauth
authfile /var/run/slim-" vt ".auth
lockfile /var/run/slim-" vt ".lock
logfile /var/log/slim-" vt ".log

# The login command.  '%session' is replaced by the chosen session name, one
# of the names specified in the 'sessions' setting: 'wmaker', 'xfce', etc.
login_cmd  exec " xinitrc " %session
sessiondir /run/current-system/profile/share/xsessions
session_msg session (F1 to change):
sessionstart_cmd " sessreg "/bin/sessreg -a -l $DISPLAY %user
sessionstop_cmd " sessreg "/bin/sessreg -d -l $DISPLAY %user

halt_cmd " shepherd "/sbin/halt
reboot_cmd " shepherd "/sbin/reboot\n"
(if (my-slim-configuration-auto-login? config)
    (string-append "auto_login yes\ndefault_user "
                   (my-slim-configuration-default-user config) "\n")
    "")
(if theme-name
    (string-append "current_theme " theme-name "\n")
    ""))))

    (define theme
      (my-slim-configuration-theme config))

    (list (shepherd-service
           (documentation "Xorg display server")
           (provision (list (symbol-append 'xorg-server-
                                           (string->symbol vt))))
           (requirement '(user-processes host-name udev))
           (start
            #~(lambda ()
                ;; A stale lock file can prevent SLiM from starting, so remove it to
                ;; be on the safe side.
                (false-if-exception (delete-file (string-append "/var/run/slim-" vt ".lock")))

                (fork+exec-command
                 (list (string-append #$(my-slim-configuration-slim config) "/bin/slim") "-nodaemon")
                 #:environment-variables
                 (list (string-append "SLIM_CFGFILE=" #$slim.cfg)
                       #$@(if theme
                              (list #~(string-append "SLIM_THEMESDIR=" #$theme))
                              #~())))))
           (stop #~(make-kill-destructor))
           (respawn? #t)))))

(define my-slim-service-type
  (service-type (name 'my-slim)
                (extensions
                 (list (service-extension shepherd-root-service-type
                                          my-slim-shepherd-service)
                       (service-extension pam-root-service-type
                                          my-slim-pam-service)

                       ;; Unconditionally add xterm to the system profile, to
                       ;; avoid bad surprises.
                       (service-extension profile-service-type
                                          (const (list xterm)))))
                (default-value (my-slim-configuration))))
